# Model Zoo

## Introduction

This file documents a collection of baselines trained with **pycls**, primarily for the [Designing Network Design Spaces](https://arxiv.org/abs/2003.13678) paper. All configurations for these baselines are located in the `configs/dds_baselines` directory. The tables below provide results and useful statistics about training and inference. Links to the pretrained models are provided as well. The following experimental and training settings are used for all of the training and inference runs.

### Experimental Settings

- All baselines were run on [Big Basin](https://code.facebook.com/posts/1835166200089399/introducing-big-basin) servers with 8 NVIDIA Tesla V100 GPUs (16GB GPU memory).
- All baselines were run using PyTorch 1.6, CUDA 10.0, and cuDNN 7.6.
- Inference times are reported for processing 64 images on 1 GPU for all models.
- Training times are reported for 100 epochs on 8 GPUs with the batch size listed.
- Each model was trained 5 times to obtain robust estimates.
- The provided models are from the runs with errors closest to the average of 5 runs.
- All models and results below are on the ImageNet-1k dataset.
- The *model id* column is provided for ease of reference.

### Reproducibility notes

- The source data for the tables below can be found in [model_complexity.json](dev/model_complexity.json), [model_error.json](dev/model_error.json), [model_timing.json](dev/model_timing.json).
- This source data is also used for unit tests on the models and helps ensure correctness and reproducibility.
- The exact code for generating the source data can be found here: [test_models.py](dev/test_models.py).
- The tables below are automatically generated by [model_zoo_tables.py](dev/model_zoo_tables.py).
- The timings below do not match the [paper](https://arxiv.org/abs/2003.13678) exactly due to the minor changes in the timing methodology.
- Most notably, the ResNe(X)t timings were faster in the paper due to use of larger batch sizes.

### Training Settings

Our primary goal is to provide simple and strong baselines that are easy to reproduce. For all models, we use our basic training settings without any training enhancements (e.g., DropOut, DropConnect, AutoAugment, EMA, etc.) or testing enhancements (e.g., multi-crop, multi-scale, flipping, etc.); please see our [Designing Network Design Spaces](https://arxiv.org/abs/2003.13678) paper for more information.

- We use SGD with momentum of 0.9, a half-period cosine schedule, and train for 100 epochs.
- For ResNet/ResNeXt/RegNet, we use a *reference* learning rate of 0.1 and a weight decay of 5e-5 (see [Figure 21](https://arxiv.org/abs/2003.13678)).
- For EfficientNet, we use a *reference* learning rate of 0.2 and a weight decay of 1e-5 (see [Figure 22](https://arxiv.org/abs/2003.13678)).
- The actual learning rate for each model is computed as (batch-size / 128) * reference-lr.
- For training, we use aspect ratio, flipping, PCA, and per-channel mean and SD normalization.
- At test time, we rescale images to (256 / 224) * train-res and take the center crop of train-res.
- For ResNet/ResNeXt/RegNet, we use the image size of 224x224 for training.
- For EfficientNet, the training image size varies following the original paper.

For 8 GPU training, we apply 5 epoch gradual warmup, following the [ImageNet in 1 Hour](https://arxiv.org/abs/1706.02677) paper. Note that the learning rate scaling rule described above is similar to the one from the [ImageNet in 1 Hour](https://arxiv.org/abs/1706.02677) paper but the number of images per GPU varies among models. To understand how the configs are adjusted, please see the examples in the `configs/lr_scaling` directory (coming soon).

## Baselines

### RegNetX Models

<table><tbody>
<!-- START RegNetX TABLE -->
<!-- TABLE HEADER -->
<th valign="bottom">model</th>
<th valign="bottom">flops<br/>(B)</th>
<th valign="bottom">params<br/>(M)</th>
<th valign="bottom">acts<br/>(M)</th>
<th valign="bottom">batch<br/>size</th>
<th valign="bottom">infer<br/>(ms)</th>
<th valign="bottom">train<br/>(hr)</th>
<th valign="bottom">error<br/>(top-1)</th>
<th valign="bottom">model id</th>
<th valign="bottom">download</th>
<!-- TABLE BODY -->
<!-- ROW RegNetX-200MF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnetx/RegNetX-200MF_dds_8gpu.yaml">RegNetX-200MF</a></td>
<td align="center">0.2</td>
<td align="center">2.7</td>
<td align="center">2.2</td>
<td align="center">1024</td>
<td align="center">11</td>
<td align="center">3.0</td>
<td align="center">31.1</td>
<td align="center">160905981</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/160905981/RegNetX-200MF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW RegNetX-400MF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnetx/RegNetX-400MF_dds_8gpu.yaml">RegNetX-400MF</a></td>
<td align="center">0.4</td>
<td align="center">5.2</td>
<td align="center">3.1</td>
<td align="center">1024</td>
<td align="center">15</td>
<td align="center">4.2</td>
<td align="center">27.4</td>
<td align="center">160905967</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/160905967/RegNetX-400MF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW RegNetX-600MF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnetx/RegNetX-600MF_dds_8gpu.yaml">RegNetX-600MF</a></td>
<td align="center">0.6</td>
<td align="center">6.2</td>
<td align="center">4.0</td>
<td align="center">1024</td>
<td align="center">18</td>
<td align="center">4.5</td>
<td align="center">25.9</td>
<td align="center">160906442</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/160906442/RegNetX-600MF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW RegNetX-800MF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnetx/RegNetX-800MF_dds_8gpu.yaml">RegNetX-800MF</a></td>
<td align="center">0.8</td>
<td align="center">7.3</td>
<td align="center">5.1</td>
<td align="center">1024</td>
<td align="center">23</td>
<td align="center">5.9</td>
<td align="center">24.8</td>
<td align="center">160906036</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/160906036/RegNetX-800MF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW RegNetX-1.6GF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnetx/RegNetX-1.6GF_dds_8gpu.yaml">RegNetX-1.6GF</a></td>
<td align="center">1.6</td>
<td align="center">9.2</td>
<td align="center">7.9</td>
<td align="center">1024</td>
<td align="center">35</td>
<td align="center">8.9</td>
<td align="center">23.0</td>
<td align="center">160990626</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/160990626/RegNetX-1.6GF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW RegNetX-3.2GF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnetx/RegNetX-3.2GF_dds_8gpu.yaml">RegNetX-3.2GF</a></td>
<td align="center">3.2</td>
<td align="center">15.3</td>
<td align="center">11.4</td>
<td align="center">512</td>
<td align="center">59</td>
<td align="center">15.1</td>
<td align="center">21.7</td>
<td align="center">160906139</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/160906139/RegNetX-3.2GF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW RegNetX-4.0GF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnetx/RegNetX-4.0GF_dds_8gpu.yaml">RegNetX-4.0GF</a></td>
<td align="center">4.0</td>
<td align="center">22.1</td>
<td align="center">12.2</td>
<td align="center">512</td>
<td align="center">74</td>
<td align="center">17.9</td>
<td align="center">21.4</td>
<td align="center">160906383</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/160906383/RegNetX-4.0GF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW RegNetX-6.4GF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnetx/RegNetX-6.4GF_dds_8gpu.yaml">RegNetX-6.4GF</a></td>
<td align="center">6.5</td>
<td align="center">26.2</td>
<td align="center">16.4</td>
<td align="center">512</td>
<td align="center">95</td>
<td align="center">24.3</td>
<td align="center">20.8</td>
<td align="center">161116590</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/161116590/RegNetX-6.4GF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW RegNetX-8.0GF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnetx/RegNetX-8.0GF_dds_8gpu.yaml">RegNetX-8.0GF</a></td>
<td align="center">8.0</td>
<td align="center">39.6</td>
<td align="center">14.1</td>
<td align="center">512</td>
<td align="center">97</td>
<td align="center">23.7</td>
<td align="center">20.7</td>
<td align="center">161107726</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/161107726/RegNetX-8.0GF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW RegNetX-12GF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnetx/RegNetX-12GF_dds_8gpu.yaml">RegNetX-12GF</a></td>
<td align="center">12.1</td>
<td align="center">46.1</td>
<td align="center">21.4</td>
<td align="center">512</td>
<td align="center">142</td>
<td align="center">34.6</td>
<td align="center">20.3</td>
<td align="center">160906020</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/160906020/RegNetX-12GF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW RegNetX-16GF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnetx/RegNetX-16GF_dds_8gpu.yaml">RegNetX-16GF</a></td>
<td align="center">15.9</td>
<td align="center">54.3</td>
<td align="center">25.5</td>
<td align="center">512</td>
<td align="center">171</td>
<td align="center">41.3</td>
<td align="center">20.0</td>
<td align="center">158460855</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/158460855/RegNetX-16GF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW RegNetX-32GF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnetx/RegNetX-32GF_dds_8gpu.yaml">RegNetX-32GF</a></td>
<td align="center">31.7</td>
<td align="center">107.8</td>
<td align="center">36.3</td>
<td align="center">256</td>
<td align="center">337</td>
<td align="center">81.8</td>
<td align="center">19.5</td>
<td align="center">158188473</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/158188473/RegNetX-32GF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- END RegNetX TABLE -->
</tbody></table>

### RegNetY Models

<table><tbody>
<!-- START RegNetY TABLE -->
<!-- TABLE HEADER -->
<th valign="bottom">model</th>
<th valign="bottom">flops<br/>(B)</th>
<th valign="bottom">params<br/>(M)</th>
<th valign="bottom">acts<br/>(M)</th>
<th valign="bottom">batch<br/>size</th>
<th valign="bottom">infer<br/>(ms)</th>
<th valign="bottom">train<br/>(hr)</th>
<th valign="bottom">error<br/>(top-1)</th>
<th valign="bottom">model id</th>
<th valign="bottom">download</th>
<!-- TABLE BODY -->
<!-- ROW RegNetY-200MF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnety/RegNetY-200MF_dds_8gpu.yaml">RegNetY-200MF</a></td>
<td align="center">0.2</td>
<td align="center">3.2</td>
<td align="center">2.2</td>
<td align="center">1024</td>
<td align="center">12</td>
<td align="center">3.3</td>
<td align="center">29.7</td>
<td align="center">176245422</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/176245422/RegNetY-200MF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW RegNetY-400MF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnety/RegNetY-400MF_dds_8gpu.yaml">RegNetY-400MF</a></td>
<td align="center">0.4</td>
<td align="center">4.3</td>
<td align="center">3.9</td>
<td align="center">1024</td>
<td align="center">19</td>
<td align="center">5.3</td>
<td align="center">25.9</td>
<td align="center">160906449</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/160906449/RegNetY-400MF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW RegNetY-600MF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnety/RegNetY-600MF_dds_8gpu.yaml">RegNetY-600MF</a></td>
<td align="center">0.6</td>
<td align="center">6.1</td>
<td align="center">4.3</td>
<td align="center">1024</td>
<td align="center">20</td>
<td align="center">5.3</td>
<td align="center">24.5</td>
<td align="center">160981443</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/160981443/RegNetY-600MF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW RegNetY-800MF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnety/RegNetY-800MF_dds_8gpu.yaml">RegNetY-800MF</a></td>
<td align="center">0.8</td>
<td align="center">6.3</td>
<td align="center">5.2</td>
<td align="center">1024</td>
<td align="center">23</td>
<td align="center">6.1</td>
<td align="center">23.7</td>
<td align="center">160906567</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/160906567/RegNetY-800MF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW RegNetY-1.6GF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnety/RegNetY-1.6GF_dds_8gpu.yaml">RegNetY-1.6GF</a></td>
<td align="center">1.6</td>
<td align="center">11.2</td>
<td align="center">8.0</td>
<td align="center">1024</td>
<td align="center">41</td>
<td align="center">10.3</td>
<td align="center">22.1</td>
<td align="center">160906681</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/160906681/RegNetY-1.6GF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW RegNetY-3.2GF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnety/RegNetY-3.2GF_dds_8gpu.yaml">RegNetY-3.2GF</a></td>
<td align="center">3.2</td>
<td align="center">19.4</td>
<td align="center">11.3</td>
<td align="center">512</td>
<td align="center">69</td>
<td align="center">17.0</td>
<td align="center">21.1</td>
<td align="center">160906834</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/160906834/RegNetY-3.2GF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW RegNetY-4.0GF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnety/RegNetY-4.0GF_dds_8gpu.yaml">RegNetY-4.0GF</a></td>
<td align="center">4.0</td>
<td align="center">20.6</td>
<td align="center">12.3</td>
<td align="center">512</td>
<td align="center">69</td>
<td align="center">17.5</td>
<td align="center">20.6</td>
<td align="center">160906838</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/160906838/RegNetY-4.0GF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW RegNetY-6.4GF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnety/RegNetY-6.4GF_dds_8gpu.yaml">RegNetY-6.4GF</a></td>
<td align="center">6.4</td>
<td align="center">30.6</td>
<td align="center">16.4</td>
<td align="center">512</td>
<td align="center">108</td>
<td align="center">27.2</td>
<td align="center">20.1</td>
<td align="center">160907112</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/160907112/RegNetY-6.4GF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW RegNetY-8.0GF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnety/RegNetY-8.0GF_dds_8gpu.yaml">RegNetY-8.0GF</a></td>
<td align="center">8.0</td>
<td align="center">39.2</td>
<td align="center">18.0</td>
<td align="center">512</td>
<td align="center">115</td>
<td align="center">29.3</td>
<td align="center">20.1</td>
<td align="center">161160905</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/161160905/RegNetY-8.0GF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW RegNetY-12GF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnety/RegNetY-12GF_dds_8gpu.yaml">RegNetY-12GF</a></td>
<td align="center">12.1</td>
<td align="center">51.8</td>
<td align="center">21.4</td>
<td align="center">512</td>
<td align="center">152</td>
<td align="center">36.9</td>
<td align="center">19.7</td>
<td align="center">160907100</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/160907100/RegNetY-12GF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW RegNetY-16GF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnety/RegNetY-16GF_dds_8gpu.yaml">RegNetY-16GF</a></td>
<td align="center">15.9</td>
<td align="center">83.6</td>
<td align="center">23.0</td>
<td align="center">256</td>
<td align="center">208</td>
<td align="center">50.8</td>
<td align="center">19.6</td>
<td align="center">161303400</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/161303400/RegNetY-16GF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW RegNetY-32GF -->
<tr>
<td align="left"><a href="configs/dds_baselines/regnety/RegNetY-32GF_dds_8gpu.yaml">RegNetY-32GF</a></td>
<td align="center">32.3</td>
<td align="center">145.0</td>
<td align="center">30.3</td>
<td align="center">256</td>
<td align="center">331</td>
<td align="center">79.4</td>
<td align="center">19.1</td>
<td align="center">161277763</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/161277763/RegNetY-32GF_dds_8gpu.pyth">model</a></td>
</tr>
<!-- END RegNetY TABLE -->
</tbody></table>

### ResNet Models

<table><tbody>
<!-- START ResNet TABLE -->
<!-- TABLE HEADER -->
<th valign="bottom">model</th>
<th valign="bottom">flops<br/>(B)</th>
<th valign="bottom">params<br/>(M)</th>
<th valign="bottom">acts<br/>(M)</th>
<th valign="bottom">batch<br/>size</th>
<th valign="bottom">infer<br/>(ms)</th>
<th valign="bottom">train<br/>(hr)</th>
<th valign="bottom">error<br/>(top-1)</th>
<th valign="bottom">model id</th>
<th valign="bottom">download</th>
<!-- TABLE BODY -->
<!-- ROW ResNet-50 -->
<tr>
<td align="left"><a href="configs/dds_baselines/resnet/R-50-1x64d_dds_8gpu.yaml">ResNet-50</a></td>
<td align="center">4.1</td>
<td align="center">25.6</td>
<td align="center">11.3</td>
<td align="center">256</td>
<td align="center">57</td>
<td align="center">14.8</td>
<td align="center">23.2</td>
<td align="center">161235311</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/161235311/R-50-1x64d_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW ResNet-101 -->
<tr>
<td align="left"><a href="configs/dds_baselines/resnet/R-101-1x64d_dds_8gpu.yaml">ResNet-101</a></td>
<td align="center">7.8</td>
<td align="center">44.5</td>
<td align="center">16.4</td>
<td align="center">256</td>
<td align="center">97</td>
<td align="center">24.0</td>
<td align="center">21.4</td>
<td align="center">161167170</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/161167170/R-101-1x64d_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW ResNet-152 -->
<tr>
<td align="left"><a href="configs/dds_baselines/resnet/R-152-1x64d_dds_8gpu.yaml">ResNet-152</a></td>
<td align="center">11.5</td>
<td align="center">60.2</td>
<td align="center">22.8</td>
<td align="center">256</td>
<td align="center">140</td>
<td align="center">33.7</td>
<td align="center">20.9</td>
<td align="center">161167467</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/161167467/R-152-1x64d_dds_8gpu.pyth">model</a></td>
</tr>
<!-- END ResNet TABLE -->
</tbody></table>

### ResNeXt Models

<table><tbody>
<!-- START ResNeXt TABLE -->
<!-- TABLE HEADER -->
<th valign="bottom">model</th>
<th valign="bottom">flops<br/>(B)</th>
<th valign="bottom">params<br/>(M)</th>
<th valign="bottom">acts<br/>(M)</th>
<th valign="bottom">batch<br/>size</th>
<th valign="bottom">infer<br/>(ms)</th>
<th valign="bottom">train<br/>(hr)</th>
<th valign="bottom">error<br/>(top-1)</th>
<th valign="bottom">model id</th>
<th valign="bottom">download</th>
<!-- TABLE BODY -->
<!-- ROW ResNeXt-50 -->
<tr>
<td align="left"><a href="configs/dds_baselines/resnext/X-50-32x4d_dds_8gpu.yaml">ResNeXt-50</a></td>
<td align="center">4.2</td>
<td align="center">25.0</td>
<td align="center">14.6</td>
<td align="center">256</td>
<td align="center">85</td>
<td align="center">22.1</td>
<td align="center">21.9</td>
<td align="center">161167411</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/161167411/X-50-32x4d_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW ResNeXt-101 -->
<tr>
<td align="left"><a href="configs/dds_baselines/resnext/X-101-32x4d_dds_8gpu.yaml">ResNeXt-101</a></td>
<td align="center">8.0</td>
<td align="center">44.2</td>
<td align="center">21.4</td>
<td align="center">256</td>
<td align="center">151</td>
<td align="center">39.9</td>
<td align="center">20.7</td>
<td align="center">161167590</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/161167590/X-101-32x4d_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW ResNeXt-152 -->
<tr>
<td align="left"><a href="configs/dds_baselines/resnext/X-152-32x4d_dds_8gpu.yaml">ResNeXt-152</a></td>
<td align="center">11.7</td>
<td align="center">60.0</td>
<td align="center">29.9</td>
<td align="center">256</td>
<td align="center">231</td>
<td align="center">57.6</td>
<td align="center">20.4</td>
<td align="center">162471172</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/162471172/X-152-32x4d_dds_8gpu.pyth">model</a></td>
</tr>
<!-- END ResNeXt TABLE -->
</tbody></table>

### EfficientNet Models

<table><tbody>
<!-- START EfficientNet TABLE -->
<!-- TABLE HEADER -->
<th valign="bottom">model</th>
<th valign="bottom">flops<br/>(B)</th>
<th valign="bottom">params<br/>(M)</th>
<th valign="bottom">acts<br/>(M)</th>
<th valign="bottom">batch<br/>size</th>
<th valign="bottom">infer<br/>(ms)</th>
<th valign="bottom">train<br/>(hr)</th>
<th valign="bottom">error<br/>(top-1)</th>
<th valign="bottom">model id</th>
<th valign="bottom">download</th>
<!-- TABLE BODY -->
<!-- ROW EfficientNet-B0 -->
<tr>
<td align="left"><a href="configs/dds_baselines/effnet/EN-B0_dds_8gpu.yaml">EfficientNet-B0</a></td>
<td align="center">0.4</td>
<td align="center">5.3</td>
<td align="center">6.7</td>
<td align="center">256</td>
<td align="center">31</td>
<td align="center">8.8</td>
<td align="center">24.9</td>
<td align="center">161305613</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/161305613/EN-B0_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW EfficientNet-B1 -->
<tr>
<td align="left"><a href="configs/dds_baselines/effnet/EN-B1_dds_8gpu.yaml">EfficientNet-B1</a></td>
<td align="center">0.7</td>
<td align="center">7.8</td>
<td align="center">10.9</td>
<td align="center">256</td>
<td align="center">47</td>
<td align="center">13.2</td>
<td align="center">24.1</td>
<td align="center">161304979</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/161304979/EN-B1_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW EfficientNet-B2 -->
<tr>
<td align="left"><a href="configs/dds_baselines/effnet/EN-B2_dds_8gpu.yaml">EfficientNet-B2</a></td>
<td align="center">1.0</td>
<td align="center">9.1</td>
<td align="center">13.8</td>
<td align="center">256</td>
<td align="center">58</td>
<td align="center">16.4</td>
<td align="center">23.5</td>
<td align="center">161305015</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/161305015/EN-B2_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW EfficientNet-B3 -->
<tr>
<td align="left"><a href="configs/dds_baselines/effnet/EN-B3_dds_8gpu.yaml">EfficientNet-B3</a></td>
<td align="center">1.8</td>
<td align="center">12.2</td>
<td align="center">23.8</td>
<td align="center">256</td>
<td align="center">92</td>
<td align="center">26.4</td>
<td align="center">22.5</td>
<td align="center">161305060</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/161305060/EN-B3_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW EfficientNet-B4 -->
<tr>
<td align="left"><a href="configs/dds_baselines/effnet/EN-B4_dds_8gpu.yaml">EfficientNet-B4</a></td>
<td align="center">4.4</td>
<td align="center">19.3</td>
<td align="center">49.5</td>
<td align="center">128</td>
<td align="center">201</td>
<td align="center">57.1</td>
<td align="center">21.4</td>
<td align="center">161305098</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/161305098/EN-B4_dds_8gpu.pyth">model</a></td>
</tr>
<!-- ROW EfficientNet-B5 -->
<tr>
<td align="left"><a href="configs/dds_baselines/effnet/EN-B5_dds_8gpu.yaml">EfficientNet-B5</a></td>
<td align="center">10.3</td>
<td align="center">30.4</td>
<td align="center">98.9</td>
<td align="center">64</td>
<td align="center">446</td>
<td align="center">122.6</td>
<td align="center">21.7</td>
<td align="center">161305138</td>
<td align="center"><a href="https://dl.fbaipublicfiles.com/pycls/dds_baselines/161305138/EN-B5_dds_8gpu.pyth">model</a></td>
</tr>
<!-- END EfficientNet TABLE -->
</tbody></table>
